#=====================================================================
# $Revision: 1.4 $
# $Date: 2003/08/22 18:23:38 $
# $Author: jnieuwen $
#
# 2003  Jeroen C. van Nieuwenhuizen and Jasper O Waale
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the license, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#======================================================================
package ipofhost ;

use strict ;
use Net::DNS ;

sub ip2hostname {
	my $hostname = shift ;
	my $dnsquery = Net::DNS::Resolver->new ;
	my $query = $dnsquery->search($hostname);
	if ($query) {
		foreach my $rr ($query->answer) {
			next unless $rr->type eq "PTR";
			return $rr->rdatastr if (not grep(/in-addr\.arpa/,$rr->rdatastr));
		}
	}
	return "" ;
}

sub mx_lookup {
	my $name = shift;
	my $res  = Net::DNS::Resolver->new;
	my @mx   = mx($res, $name);
	if (@mx) {
		foreach my $rr (@mx) {
			return $rr->exchange ;
		}
	}
	return "";
}

sub ip2hostname_has_mx {
	my $hostname = shift ;
	$hostname = ip2hostname($hostname) ;
	$hostname =~ s/\.$//;
	my @points = split(/\./,$hostname);
	while($#points > 2) {
		$hostname =~ s/^[^\.]*\.// ;
		@points = split(/\./,$hostname);
	}
	while ($hostname ne "") {
		my $res = mx_lookup($hostname) ;
		return "abuse@".$hostname if ($res ne "") ;
		$hostname =~s/^[^\.]*\.// ;
	}
	return "" ;
}


# Given a hostname converts this to a ip number.
sub hostname2ip {
	my $hostname = shift ;
	my $dnsquery = Net::DNS::Resolver->new ;
	if ($hostname ne "") {
		my $query = $dnsquery->search($hostname);
		if ($query) {
			foreach my $rr ($query->answer) {
				next unless $rr->type eq "A";
				return $rr->address;
			}
		}
	}
	return "" ;
}

# Given an IP-number or hostname returns the IP-number.
sub getip {
	my $host = shift ;
	$host = hostname2ip($host) if (grep(/[a-zA-Z]/,$host)) ;
	return $host ;
}

1;
