#! /usr/bin/perl
#================================================
# $Id: log2html.pm,v 1.1 2003/04/11 19:11:30 jnieuwen Exp $
# $Revision: 1.1 $
# $Author: jnieuwen $
# 2003  Jeroen C. van Nieuwenhuizen
#================================================
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the license, or 
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#================================================

package log2html ;

use POSIX qw(strftime);

=head1 DESCRIPTION

log2html provides the functions necessary to convert a squidefender log file
to its html version.

=head1 USES

B<POSIX qw(strftime)>

=head1 FUNCTIONS

=cut

my $color = 1 ;

=head2 write_table_row (F<file reference>, I<time of mail>, I<time of attack>,
I<attacking host>, I<address we send mail to>, I<No of attacks>)

Writes a row of a table to the html file referenced to by F<file reference>.

=cut
sub write_table_row {
	my $outfile = shift ;
	my $mailsend = shift ;
	my $attack = shift ;
	my $host = shift ;
	my $isp = shift ;
	my $count = shift ;
	chomp $count ;
	$mailsend =~ s/Mail send: // ;
	$attack =~ s/Attack on: // ;
	$isp =~ s/Mail send to: // ;
	if ($color) {
		print $outfile <<EOF
<tr>
<td style="background: #e0e0e0 ; color: black ; font-family: Times ; padding: 6px ; vertical-align: top;">$mailsend</td>
<td style="background: #e0e0e0 ; color: black ; font-family: Times ; padding: 6px ; vertical-align: top;">$attack</td>
<td style="background: #e0e0e0 ; color: black ; font-family: Times ; padding: 6px ; vertical-align: top; ">$host</td>
<td style="background: #e0e0e0 ; color: black ; font-family: Times ; padding: 6px ; vertical-align: top; ">$isp</td>
<td style="background: #e0e0e0 ; color: black ; font-family: Times ; padding: 6px ; vertical-align: top; ">$count</td>
</tr>
EOF
		;
		$color = 0;
	} else {
		print $outfile <<EOF
<tr>
<td style="background: #C0C0C0 ; color: black ; font-family: Times ; padding: 6px ; vertical-align: top;">$mailsend</td>
<td style="background: #C0C0C0 ; color: black ; font-family: Times ; padding: 6px ; vertical-align: top;">$attack</td>
<td style="background: #C0C0C0 ; color: black ; font-family: Times ; padding: 6px ; vertical-align: top;">$host</td>
<td style="background: #C0C0C0 ; color: black ; font-family: Times ; padding: 6px ; vertical-align: top;">$isp</td>
<td style="background: #C0C0C0 ; color: black ; font-family: Times ; padding: 6px ; vertical-align: top;">$count</td>
</tr>
EOF
		;
		$color = 1;
	}
}

=head2 print_page_start (F<file reference>)

Writes the html code to start a page to the file referenced by F<file reference>.

=cut
sub print_page_start {
	my $outfile = shift;
	print $outfile <<EOF
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">
<html>
<head><meta http-equiv="Content-Type" content="text/html; charset=ISO-8859-1">
  <title>squidefender log</title>
  <meta name="Description" content="squidefender log output">
  <meta name="KeyWords" content="squidefender output">
  <meta NAME="ROBOTS" CONTENT="NOARCHIVE">
</head>

<body style="{ background: #cfcfcf; color: black;" } >

<div style="
 color: #357784 ;
 background: #cfcfcf;
 font-family: Verdana;
 font-style: italic;
 font-weight: bold ;
 font-size: larger;
 text-align: left ;
 line-height: 2.0 ;
 display: table ;
 padding: 5px ; ">squidefender log file</div>
<table cellspacing=0 cellpadding=5 border=0 >
<tr style="{ color: white ; background: black ; }" >
	<td style="color: white ; font-family: Arial ; padding: 6px ; font-weight: bold; background: black">Mail send at</td>
	<td style="color: white ; font-family: Arial ; padding: 6px ; font-weight: bold; background: black">Attack at</td>
	<td style="color: white ; font-family: Arial ; padding: 6px ; font-weight: bold; background: black">Attacking Host</td>
	<td style="color: white ; font-family: Arial ; padding: 6px ; font-weight: bold; background: black">Mail send to</td>
	<td style="color: white ; font-family: Arial ; padding: 6px ; font-weight: bold; background: black">No. of attacks</td>
</tr>
EOF
	;
}

=head2 print_page_end (F<file reference>)

Writes the html code to end a page to the file referenced by F<file reference>.

=cut
sub print_page_end {
	my $outfile = shift;
	my $time = strftime("%d/%b/%Y %H:%M %Z",localtime(time));
	print $outfile <<EOF
</table>
<p>
<hr>
<font size=-2>Generated by 
<a href="http://kabel.netvisit.nl/~nieuwe03/squidefender.html">squidefender</a> on
$time
</font>
<hr>
  </body>
</html>
EOF
	;
}

=head2 create_html_log (F<squidefenderlogfile>, F<htmlfile>)

Reads the logdata from F<squidefenderlogfile> and writes it in htmlformat
to F<htmlfile>.

=cut
sub create_html_log {
	my $inputfile = shift ;
	my $output = shift ;
	if (open(FILE,$inputfile) && open($outfile,">".$output)) {
		print_page_start($outfile) ;
		while (<FILE>) {
			write_table_row($outfile,split(/\t/,$_));
		}
		close FILE;
		print_page_end($outfile);
		close $outfile ;
		return (1)
	} else {
		return 0 ;
	}
}

1 ;
