#! /usr/bin/perl -w
#=====================================================================
# $Revision: 1.18 $
# $Date: 2003/09/23 19:08:59 $
# $Author: jnieuwen $
#
# 2003  Jeroen C. van Nieuwenhuizen and Jasper O Waale
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the license, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#======================================================================
use strict ;

# Go to the script directory so we can do all requires.
my $path = $0;
$path =~ s#/[^/]*$## ;
chdir($path);

#The modules we need.
require "warnisp.pm" ;
require "ipofhost.pm" ;
require "whois2address.pm" ;
require "log2html.pm" ;
require "wormdb.pm" ;

use POSIX qw(strftime);
use File::Copy ;

#================================================
#The globals we need.
my %g_options ; # A hash reference which is going to contain all options in the future.
my %line ; #A hash reference with the host as key to all attack lines found.
my %count ; #A hash reference with the host as key to the number of attacks of host.
my %patterns ; #The patterns we look for. (Key is ID)
my %offence ; #The ID of the offence we found. (Key is host)
my %templates ; #Points to the templates of the messages. (Key is ID)
my %time ; #Contains the time of the last attack of host (Key is host)
my @ipignore ; #Contains regular expressions about the hosts we want to ignore.
my $offset = 0 ; # The initial value of the timestamp.
my %blocktime ; #Contains the blocktime for a given ID.

#TODO: Clean up variables below.
my $squidlog ; #Will contain the path to the squidlog file.
my $timestampfile ; #Will contain the path to the timestamp file.
my $ourhost ; #Will contain the name of the host we are running on.
my $from ; #The address of the user that sends the warning mails.
my $logfile ; #The file we write our logfile to.
my $firewallcommand ; #The command we execute for each host.
my $mailhost ; #The host we use to send mail.
my $bcc ="" ; #A comma seperated list of the BCC receivers.
my $cc = "" ; #A comma seperated list of the CC receivers.
#All one dimensional options that exists.
my %g_list = (
	backup => 1, backupfile => 1, database => 1, isp_pending => 1, htmlout => 1,
	ipinfo => 1, subject => 1, firewallremove => 1
) ;

#================================================
#Converts a addres in the form a@a.a to a [at] a [dot] a
sub create_result {
	my $address = shift ;
	$address =~ s/@/ [at] /g ;
	$address =~ s/\./ [dot] /g ;
	return "Mail send to: ".$address ;
}
#================================================

#================================================
#writes a line to the log file.
sub write_log_file {
	my $file = shift ;
	my $ip = shift ;
	my $result = shift ;
	my $count = shift ;
	my $date_string = strftime("Mail send: %d/%b/%Y %H:%M\t",localtime(time));
	my $attack_date = strftime("Attack on: %d/%b/%Y %H:%M\t",localtime($time{$ip}));
	if (open(FILE,">>".$file)) {
		print FILE $date_string.$attack_date.$ip."\t".$result."\t".$count."\n" ;
		close FILE ;
	} else {
		warn "Could not open logfile for writing (".__PACKAGE__." at ".__LINE__.")\n" ;
	}
}
#================================================

#================================================
# Attack check routine.
sub handle_attack {
	my $line = shift ;
	my $timestamp = shift ;
	my $client = shift ;
	foreach my $id (keys %patterns) {
		my $pat = $patterns{$id} ;
		if (grep(/$pat/,$line)) {
			#Damm we are under attack. Add all information about attack.
			my $datetime = strftime("[%d/%b/%Y %H:%M %z] ",localtime($timestamp));
			$line{$client} .= "\n".$datetime.$line if (defined $line{$client}) ;
			$line{$client} = $datetime.$line unless (defined $line{$client}) ;
			$count{$client}++ if (defined $count{$client});
			$count{$client} = 1 unless (defined $count{$client});
			$offence{$client} = $id ;
			$time{$client} = $timestamp ;
		}
	}
}
#================================================

#================================================
#Checks whether a line from the squide logfile contains an attack.
sub split_line {
	my $line = shift ;
	my $offset = shift ;
	$line =~ s/[ \t]+/ /g ;
	chomp $line ;
	#Does the line contain an ipaddress. If not no attack.
	if (grep(/ [0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3} /,$line)) {
		my ($timestamp,$elapsed,$client,$logtag,$size,$request,$url,$ident,$hostname,$contenttype) = split(/ /,$line,10);
		# Is it an attack from hosts we want to ignore.
		foreach my $ip (@ipignore) {
			return $timestamp if (grep(/$ip/,$client)); #Yes, exit.
		}
		#Nope. Attack occured after last run?
		if ($timestamp > $offset) {
			handle_attack($line, $timestamp, $client) ;
			return $timestamp ; #parsed a line succesfull so we need to give back the newtimestamp.
		}
	}
	return 0 ;# Nothing found so keep the old timestamp.
}
#================================================

#================================================
#reads the data from the squidlogfile and hands it over for checking.
sub find_offending_hosts {
	my $logfile = shift ;
	my $offset = shift ;
	my $newoffset = 0;
	if (open(FILE,$logfile)) {
		while (my $line = <FILE>) {
			$newoffset = split_line($line,$offset) ;
		}
		close FILE ;
	} else {
		warn "Log file not found\n";
	}
	return $newoffset if ($newoffset > $offset) ; #If we have a recenter timestamp. Use it.
	return $offset ; #No newer timestamp. Use the old one.
}
#================================================

#================================================
#Splits a line of a configfile if it is a configline.
sub split_config_line {
	return ($1,$2) if ($_[0] =~ /^\s*([^\s#=]+)=([^\s#](\s*[^\s#]+)*)\s*$/ ) ;
	return ("","");
}
#================================================

#================================================
# Reads the data from the specified config file.
sub read_config_file {
	if (open(CONFIGFILE,$_[0])) {
		while(<CONFIGFILE>) {
			my ($name, $value) = split_config_line($_);
			$squidlog = $value if ($name eq "squidlog");
			$timestampfile = $value if ($name eq "timestamp");
			$ourhost = $value if ($name eq "host");
			$from = $value if ($name eq "from");
			$logfile = $value if ($name eq "logfile");
			$firewallcommand = $value if ($name eq "firewallcommand");
			$mailhost = $value if ($name eq "mailhost");
			$bcc = $value if ($name eq "bcc");
			$cc = $value if ($name eq "cc");
			$g_options{$name} = $value if (exists $g_list{$name}) ;
			push (@ipignore,$value) if ($name eq "ipignore");
			if ($name eq "blocktime") {
				my ($id,$idvalue) = split_config_line($value);
				$blocktime{$id} = $idvalue ;
			}
			if ($name eq "pattern") {
				my ($id,$idvalue) = split_config_line($value);
				$patterns{$id} = $idvalue ;
			}
			if ($name eq "template") {
				my ($id,$idvalue) = split_config_line($value);
				$templates{$id} = $idvalue ;
			}
		}
		close CONFIGFILE ;
	} else {
		die "Error: reading configfile\n" ;
	}
}
#================================================

#================================================
# Executes the adaption of the firewall.
sub adapt_firewall {
	my $host = ipofhost::getip($_[0]) ;
	my $temp = $firewallcommand ;
	$temp =~ s/\$IP\$/$host/g;
	if (system($temp)) {
		print STDERR "Could not execute command $temp\n" ;
		return 0 ;
	}
	return 1 ;
}
#================================================
# Removes an IP from the firewall.
sub remove_firewall {
	my $host = ipofhost::getip($_[0]) ;
	my $temp = $g_options{firewallremove} ;
	$temp =~ s/\$IP\$/$host/g;
	if (system($temp)) {
		print STDERR "Could not execute command $temp\n" ;
		return 0 ;
	}
	return 1 ;
}
#================================================

#================================================
sub make_backup {
	if ($g_options{'backup'} && -e $g_options{'backupfile'}) {
		File::Copy::copy($g_options{'backupfile'},$g_options{'backupfile'}.".bak") ;
	}
}
#================================================

#================================================
sub restore_backup {
	if ($g_options{'backup'} && -e $g_options{'backupfile'}.".bak") {
		File::Copy::copy($g_options{'backupfile'}.".bak",$g_options{'backupfile'}) ;
	}
}
#================================================

#================================================
# Check and if necessary restore the backup.
sub restore_backup_on_failure {
	my @new_size = stat($g_options{'backupfile'}) ;
	my @old_size = stat($g_options{'backupfile'}.".bak") ;
	if ($g_options{'backup'} && -e $g_options{'backupfile'}.".bak" &&
		$new_size[7] < $old_size[7] ) {
		File::Copy::copy($g_options{'backupfile'}.".bak",$g_options{'backupfile'}) ;
		return 1 ;
	}
	return 0 ;
}
#================================================

#================================================
# Send warning message
sub send_warning_message {
	my $host = shift ;
	my $abuse = whois2address::find_address_to_warn($host,$g_options{ipinfo});
	if ($abuse ne "") {
		my $message = warnisp::message($ourhost,$host,$count{$host},$line{$host},$templates{$offence{$host}}) ;
		if ($message ne "") {
			if(warnisp::send_message($abuse,$from,$g_options{subject},$message,$mailhost,$bcc,$cc)) {
				write_log_file($logfile,$host,create_result($abuse),$count{$host}) ;
			} else {
				warn "Sending of mail about ".$host." failed\n" ;
				write_log_file($logfile,$host,"Sending of mail failed",$count{$host}) ;
			}
		} else {
			write_log_file($logfile,$host,"Could not find message file",$count{$host}) ;
		}
		return $abuse ;
	} else {
		write_log_file($logfile,$host,"No address found",$count{$host}) ;
	}
	return "" ;
}
#================================================

#================================================
# MAIN PROGRAM
#================================================
# Read the configuration file.
read_config_file("squidefender.conf") ;
$ourhost = (gethostbyname 'localhost')[0] if (not defined $ourhost);
$mailhost = $ourhost if (not defined $mailhost);

# Get a lock on the lock file to be sure
# we are the only one running.
open(LOCKFILE,">lock.file") or die "Can not create lockfile.\n" ;
if (flock(LOCKFILE, 2 | 4) == 0) {
	# We did not get the lock so exit.
	close LOCKFILE ;
	exit ;
}

# Read the old timestamp.
open (FILE,$timestampfile) or die "No timestamp file\n" ;
$offset = <FILE> ;
close FILE ;
chomp $offset ;

# Find all offending hosts.
$offset = find_offending_hosts($squidlog,$offset)."\n";

#Warn the isp for each attacking host.
foreach my $host (keys %line) {
	my $database = wormdb->new($g_options{'database'},$host) ;
	$database->read();
	for (my $tel = 0;$tel < $count{$host};$tel++) {
		$database->attack();
	}
	if ($database->isp_warned()) {
		if (!$database->isp_pending($g_options{'isp_pending'})) {
			$database->{address} = send_warning_message($host) ;
			$database->last_isp_warning_now() ;
		}
	} else {
		$database->{address} = send_warning_message($host) ;
		$database->last_isp_warning_now() ;
	}
	#Set the blocktime
	if (defined $blocktime{$offence{$host}}) {
		$database->set_block_until($blocktime{$offence{$host}}) ;
	} else {
		#default one year.
		$database->set_block_until(365) ;
	}
	# Execute a command for to block the attacking host if specified.
	if (defined $firewallcommand) {
		unless ($database->{active}) {
			make_backup();
			$database->set_active() ;
			if (not adapt_firewall($host)) {
				restore_backup();
				$database->reset_active() ;
			} else {
				$database->reset_active() if (restore_backup_on_failure());
			}
		}
	}
	#finally write the database.
	$database->write();
}

# Write the new offset.
open(FILE,">".$timestampfile) or die "Could not open timestamp file for writing\n";
print FILE $offset."\n";
close FILE ;
print $g_options{htmlout}."\n" ;

#Remove expired entries from the firewall.
if (defined $g_options{firewallremove}) {
	#Dummy database entry will never be written.
	my $database = wormdb->new($g_options{'database'},'127.0.0.2') ;
	foreach my $host ($database->return_all_expired()) {
		my $dbtemp = wormdb->new($g_options{'database'},$host) ;
		$dbtemp->read();
		if (remove_firewall($host)) {
			$dbtemp->reset_active() ;
			$dbtemp->write();
		}
	}
}

#TODO: Add firewall optimalization code here.

#Does the user want html output.
if (defined $g_options{htmlout}) {
	log2html::create_html_log($logfile,$g_options{htmlout});
}

# release our lock on the lockfile.
flock(LOCKFILE, 8) ;
close LOCKFILE ;

#================================================

=head1 AUTHOR

Written by Jeroen van Nieuwenhuizen and Jasper O Waale.

=head1 VERSION

$Revision: 1.18 $ $Date: 2003/09/23 19:08:59 $

$Source: /home/jnieuwen/cvsrepos/squid/squidefender.pl,v $

=head1 REPORTING BUGS

Report bugs to <jnieuwen@softhome.net>

=head1 COPYRIGHT

Copyright (C) 2003 by Jeroen van Nieuwenhuizen and Jasper O Waale. Distributed
under the GNU GPL. See the file "F<LICENSE>", supplied with the distribution
for additional information.

=cut
