#=====================================================================
# $Revision: 1.4 $
# $Date: 2003/08/22 18:23:38 $
# $Author: jnieuwen $
#
# 2003  Jeroen C. van Nieuwenhuizen and Jasper O Waale
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the license, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#======================================================================

package whois2address ;

use strict ;
use Socket ;
use IO::Socket ;
use Net::DNS ;
require "ipofhost.pm" ;

# Gets the whois data from a whois server.
sub get_whois_data {
	my $server = shift ;
	my $ipadress = shift ;
	my $result = "" ;
	my $count = 0 ;
	my $line ;
	if (my $socket = IO::Socket::INET->new(PeerAddr => $server, PeerPort => 43, Proto => 'tcp')) {
		$socket->send($ipadress."\n",0);
		$result .= $line while (defined($line = <$socket>) && ($count++ < 500)) ;
		$socket->close();
	}
	return $result ;
}

# Checks whether the whois data contains
# a pointer to a more specific whois server.
sub contains_whois_server {
	my $data = shift ;
	my $server = shift ;
	my @servers ;
	my $old_server = $$server ;
	if (grep(/whois\./i,$data)) {
		my $rule ;
		my @lines = split(/\n/i,$data);
		foreach $rule (@lines) {
			if ($rule =~ /[ ](whois\.[^\s]*)/i) {
				my $whois_temp = $1 ;
				$whois_temp =~ s/\.$// ;
				push (@servers,$whois_temp) ;
			}
		}
		if ($#servers > -1) {
			foreach my $item (@servers) {
				$item =~ s/[^a-zA-Z\.]//; #Remove strange characters.
				$$server = $item if ($item ne $old_server) ;
			}
		}
		return 1 if ($$server ne $old_server); 
	}
	return 0;
}

# finds the most specific whois data for a given ip address.
sub find_whois_data {
	my $whoisserver = shift ;
	my $ip = shift ;
	my $data ;
	my $continue = 1;
	while ($continue) {
		$data = get_whois_data($$whoisserver,$ip);
		$continue = contains_whois_server($data,$whoisserver);
		if ($continue == 0 && $$whoisserver eq "whois.arin.net"){
			my @lines = split(/\n/,$data);
			my $line ;
			my $netdata = "";
			foreach $line (@lines) {
				if (grep(/\(NET-[0-9]{1,3}-[0-9]{1,3}/,$line)) {
					my $garbage ;
					($garbage,$netdata) = split(/\(NET/,$line,2) ;
					$netdata =~s/\)//;
					$netdata = "NET".$netdata ;
					$data .= get_whois_data($$whoisserver,$netdata) if ($netdata ne "") ;
					print "here\n";
				}
			}
		}
	}
	return $data ;
}

# Given an IP and a ipinfo file finds
# the whois server to use.
sub start_server {
	my $ip = shift ;
	my $ipinfo = shift ;
	my $garb ;
	my $result = "whois.arin.net" ; #The default.
	$ip =~ s/\..*// ;
	$ip= "^".$ip." ";
	if (open(IPDATA,$ipinfo)) {
		while (<IPDATA>) {
			($garb,$result)=split(/ /,$_,2) if (/$ip/);
		}
		close IPDATA ;
	}
	chomp $result ;
	return $result ;
}

sub email_has_mx {
	my $email = shift ;
	$email =~ s/^[^@]*@//;
	return "" if ($email eq "");
	return ipofhost::mx_lookup($email) if (ipofhost::mx_lookup($email) ne "");
	#return ipofhost::hostname2ip($email) if (ipofhost::hostname2ip($email) ne "");
	return "" ;
}

sub names_match {
	my $first = shift ;
	my $second = shift ;
	my $count = shift;
	my $temp ;
	($temp,$first) = split(/@/,$first,2) ;
	($temp,$second) = split(/@/,$second,2) ;
	$second = substr($second,1) while (length($second) > length($first)) ;
	$first = substr($first,1) while (length($first) > length($second)) ;
	while ($first ne $second) {
		$second = substr($second,1) ;
		$first = substr($first,1) ;
	}
	return length($first);
}

sub find_abuse_net {
	my $ip = shift ;
	my $result = "" ;
	my $domain = ipofhost::ip2hostname($ip);
	my ($res, $query, @r);
	my $cont = 1 ;

	$domain =~ s/\.$// ;

	return "" if ($domain eq "");

	$res = new Net::DNS::Resolver;
	while($cont) {
		$query = $res->search("$domain.contacts.abuse.net", "TXT");
		if ($query) {
			foreach my $rr ($query->answer) {
				push(@r, $rr->txtdata) if ($rr->type eq "TXT") ;
			}
			$cont = 0 ;
		} else { # Net::DNS rejects special characters, strip off
			# subdomains and see if a parent domain works
			if($domain =~ m{^[^.]+\.([^.]+\..+)}) {
				$domain = $1;
			} else {
				$cont = 0;
			}
		}
	}
	my $match_count = 0; #Match count.
	foreach my $abuse (@r) {
		if (grep(/^abuse@/,$abuse) && email_has_mx($abuse) ne "") {
			my $temp_match = names_match($abuse,"bla@".$domain);
		 	if ($temp_match > $match_count) {
				$result = $abuse ;
				$match_count = $temp_match ;
			}
		}
	}
	if ($result eq "") {
		$match_count = 0;
		foreach my $email (@r) {
			if (email_has_mx($email) ne "") {
				my $temp_match = names_match($email,"bla@".$domain);
				if ($temp_match > $match_count) {
					$result = $email ;
					$match_count = $temp_match ;
				}
			}
		}
	}
	return $result ;
}

# Finds the email address to send warning messages to.
# The returned email address which is returned is the following:
# if abuse@ exists it returns it.
# else if security@ exists it returns it.
# else if a techemail address is listed it returns it.
# else if a hostmaster@ exists it returns it.
# else return the empty string.
sub find_address_to_warn {
	my $ip = shift ;
	my $ipinfo = shift ;

	#get the ip number of the host.
	eval {
		local $SIG{ALRM} = sub {die "alarm" };
		alarm(120) ;
		$ip = ipofhost::getip($ip) ;
		alarm(0) ;
	} ;
	if ($@) {
		die unless $@ =~ /alarm/ ;
		return "" ;
	}

	#Do a save query to the abuse.net database.
	my $abuse_net = "";
	eval {
		#First use the abuse.net database.
		local $SIG{ALRM} = sub {die "alarm" };
		alarm(120) ;
		$abuse_net = find_abuse_net($ip) ;
		alarm(0);
	} ;
	if ($@) {
		die unless $@ =~ /alarm/ ;
	} else {
		return $abuse_net if ($abuse_net ne "") ;
	}

	#Failed. Try whois lookups.
	my $addr = "";
	eval {
		local $SIG{ALRM} = sub {die "alarm" };
		alarm(120) ;
		my $whoisserver = start_server($ip,$ipinfo) ;
		my $data = find_whois_data(\$whoisserver,$ip);
		my @lines = split(/\n/,$data) ;
		my @address = ("","","","","","");
		my $rule ;
			foreach $rule (@lines) {
			chomp $rule ;
			if (grep(/[ :<](abuse|security|hostmaster)@/i,$rule)) {
				my @addresses = split(/ /,$rule) ;
				my $line ;
				foreach $line (@addresses) {
					$address[0] = $line if (grep(/abuse@/,$line)) ;
					$address[1] = $line if (grep (/security@/,$line)) ;
					$address[4] = $line if (grep (/hostmaster@/,$line)) ;
				}
			}
			if (grep(/^TechEmail:/,$rule) && $whoisserver eq "whois.arin.net") {
				$rule =~s/TechEmail://g;
				$address[2] = $rule ;
			}
			if (grep(/^e-mail:/,$rule) && ($whoisserver eq "whois.ripe.net" || $whoisserver eq "whois.apnic.net")) {
				$rule =~s/e-mail://g;
				$address[3] = $rule ;
			}
		}
		@address = reverse @address ;
		$addr = pop @address ;
		$addr =~ s/mailto://;
		$addr =~ s/[<> ]//g;
		while (($addr eq "" || email_has_mx($addr) eq "") && $#address > -1) {
			$addr = pop @address ;
			$addr =~ s/mailto://;
			$addr =~ s/[<> ]//g;
		}

		$whoisserver =~ s/^whois\.//g;
		$whoisserver =~ s/\./\./g;

		# Avoid addresses belonging to the whois server domain.
		# They are probably not the ones that should be warned.
		$addr = "" if (grep(/$whoisserver/,$addr));
		alarm(0);
	} ;
	if ($@) {
		die unless $@ =~ /alarm/ ;
	} else {
		return $addr if (email_has_mx($addr) ne "") ;
	}
	return "" ;
}

1;
